@@ -1,12 +1,13 @@
-from contextlib import contextmanager
-from collections import namedtuple
-from subprocess import check_output, CalledProcessError
-from os.path import join as p, exists, split as split_path, isdir, isabs
-from textwrap import dedent
-import tempfile
-import shutil
-import shlex
+import json
 import os
+from os.path import join as p, exists, split as split_path, isdir, isabs
+import shlex
+import shutil
+import tempfile
+from collections import namedtuple
+from contextlib import contextmanager
+from subprocess import check_output, CalledProcessError
+from textwrap import dedent
 
 from owmeta_core.command import OWM
 from owmeta_core.command_util import DEFAULT_OWM_DIR
@@ -18,13 +19,15 @@ from pkg_resources import resource_stream
 from pytest import fixture, mark
 
 
-__version__ = '0.0.6'
+__version__ = '0.0.7'
+
+BundleData = namedtuple('BundleData', ('id', 'version', 'source_directory', 'remote'),
+                        defaults=dict(source_directory=None, remote=None))
 
-BundleData = namedtuple('BundleData', ('id', 'version', 'source_directory', 'remote'))
 TEST_BUNDLES_DIRECTORY = os.environ.get('TEST_BUNDLES_DIRECTORY', 'bundles')
 
 
-def bundle_fixture_helper(bundle_id, version=None):
+def bundle_fixture_helper(id, version=None):
     '''
     Creates test fixtures for testing with pre-made bundles.
 
@@ -32,7 +35,7 @@ def bundle_fixture_helper(bundle_id, version=None):
 
     Parameters
     ----------
-    bundle_id : str
+    id : str
         The ID of the bundle
     version : int, optional
         The version of the bundle
@@ -44,19 +47,24 @@ def bundle_fixture_helper(bundle_id, version=None):
     '''
     def bundle(request):
         # Raises a BundleNotFound exception if the bundle can't be found
-        nonlocal version, bundle_id
-        if bundle_id is None and version is None:
+        nonlocal version, id
+
+        if id is None and version is None:
             try:
-                bundle_id, version = request.param
+                bundle_id, bundle_version = request.param
             except AttributeError as e:
                 raise Exception('Use the bundles decorator to declare bundle'
                         ' versions for this test') from e
         elif version is None:
             try:
-                version = request.param
+                bundle_version = request.param
+                bundle_id = id
             except AttributeError as e:
                 raise Exception('Use the bundle_versions decorator to declare bundle'
                         ' versions for this test') from e
+        else:
+            bundle_id = id
+            bundle_version = version
 
         remotes = []
         marker = request.node.get_closest_marker("bundle_remote")
@@ -74,13 +82,15 @@ def bundle_fixture_helper(bundle_id, version=None):
                 remote = retrieve_remote_by_name(p(DEFAULT_OWM_DIR, 'remotes'), test_bundles_remote)
                 if remote:
                     remote_defined = True
-                    remotes.append(remote)
+                else:
+                    raise
             else:
                 remote_defined = True
+            remotes.append(remote)
 
         TestBundleLoader = None
         try:
-            source_directory = find_bundle_directory(TEST_BUNDLES_DIRECTORY, bundle_id, version)
+            source_directory = find_bundle_directory(TEST_BUNDLES_DIRECTORY, bundle_id, bundle_version)
         except BundleNotFound:
             if not remote_defined:
                 raise
@@ -100,7 +110,7 @@ def bundle_fixture_helper(bundle_id, version=None):
                     pass
 
                 def bundle_versions(self):
-                    return [version]
+                    return [bundle_version]
 
                 @classmethod
                 def can_load_from(cls, ac):
@@ -109,7 +119,7 @@ def bundle_fixture_helper(bundle_id, version=None):
                     return False
 
                 def can_load(self, ident, version):
-                    return ident == bundle_id and version == version
+                    return ident == bundle_id and version == bundle_version
 
                 def load(self, ident, version):
                     shutil.copytree(source_directory, self.base_directory)
@@ -119,7 +129,7 @@ def bundle_fixture_helper(bundle_id, version=None):
 
         yield BundleData(
                 bundle_id,
-                version,
+                bundle_version,
                 source_directory,
                 remotes)
         if TestBundleLoader:
@@ -208,12 +218,44 @@ def _owm_project_helper(request):
                 return r
 
             def fetch(bundle_data):
+                '''
+                Fetch the bundle described by the given data into the test home directory
+
+                Parameters
+                ----------
+                bundle_data : .BundleData
+                    The bundle description
+                '''
                 bundles_directory = p(res.test_homedir, '.owmeta', 'bundles')
                 fetcher = Fetcher(bundles_directory, bundle_data.remote)
                 return fetcher.fetch(bundle_data.id, bundle_data.version)
 
+            _fetch = fetch
+
+            def add_dependency(bundle_data, fetch=True):
+                '''
+                Add a bundle dependency to the project. By default, this will also fetch
+                the bundle.
+
+                Parameters
+                ----------
+                bundle_data : .BundleData
+                    The bundle description
+                fetch : bool, optional
+                    If `True` fetch the bundle into the project's test directory.
+                    Default is `True`.
+                '''
+                orig_deps = owm().config.get('dependencies')
+                deps = [{'id': bundle_data.id, 'version': bundle_data.version}]
+                if orig_deps is not None:
+                    deps = orig_deps + deps
+                owm().config.set('dependencies', json.dumps(deps))
+                if fetch:
+                    _fetch(bundle_data)
+
             res.owm = owm
             res.fetch = fetch
+            res.add_dependency = add_dependency
 
             yield res
         finally:
